"""
Assets module for Helmut4 client.
"""

from typing import Any, Dict, List, Optional

from ._base import BaseModule


class AssetsModule(BaseModule):
    """Module for managing assets"""

    def search(
        self,
        search_filter: Optional[Dict] = None,
        limit: int = 500,
        page: int = 0,
        project_id: Optional[str] = None,
    ) -> List[Dict]:
        """
        Search for assets based on a filter.

        Args:
            search_filter: Search filter criteria
            limit: Maximum number of results
            page: Page number for pagination
            project_id: Optional project ID to filter by

        Returns:
            List of matching asset objects
        """
        params: Dict[str, Any] = {"limit": limit, "page": page}
        if project_id:
            params["projectId"] = project_id

        return self.client.request(
            "POST", "/v1/co/search", params=params, json=search_filter or {}
        )

    def get_by_id(self, asset_id: str) -> Dict:
        """
        Get an asset by ID.

        Args:
            asset_id: The ID of the asset

        Returns:
            Asset object
        """
        return self.client.request(
            "GET", "/v1/co/assets", params={"assetId": asset_id}
        )

    def get_by_project_id(self, project_id: str) -> List[Dict]:
        """
        Get all assets for a project.

        Args:
            project_id: The ID of the project

        Returns:
            List of asset objects
        """
        return self.client.request("GET", f"/v1/co/assets/{project_id}")

    def create(
        self, asset_data: Dict, create_bin_structure: bool = True
    ) -> Dict:
        """
        Create a new asset.

        Args:
            asset_data: Asset data
            create_bin_structure: Whether to create parent bins according to
                assetBreadCrumb

        Returns:
            Created asset object
        """
        params = {"createBinStructure": str(create_bin_structure).lower()}
        return self.client.request(
            "POST", "/v1/co/assets", params=params, json=asset_data
        )

    def update(self, asset_data: Dict) -> Dict:
        """
        Update an existing asset.

        Args:
            asset_data: Asset data including the ID

        Returns:
            Updated asset object
        """
        return self.client.request("PUT", "/v1/co/assets", json=asset_data)

    def patch(self, asset_data: Dict) -> Dict:
        """
        Update specific fields of an existing asset.

        Args:
            asset_data: Asset data with the fields to update (must include ID)

        Returns:
            Updated asset object
        """
        return self.client.request("PATCH", "/v1/co/assets", json=asset_data)

    def delete(
        self, asset_ids: List[str], project_id: Optional[str] = None
    ) -> Dict:
        """
        Delete one or more assets.

        Args:
            asset_ids: List of asset IDs to delete
            project_id: Optional project ID the assets belong to

        Returns:
            Result object
        """
        params = {}
        if project_id:
            params["projectId"] = project_id

        return self.client.request(
            "DELETE", "/v1/co/assets", params=params, json=asset_ids
        )

    def set_metadata(
        self,
        asset_id: str,
        metadata: List[Dict],
        project_id: Optional[str] = None,
        delete: bool = False,
    ) -> Dict:
        """
        Set metadata for an asset.

        Args:
            asset_id: ID of the asset
            metadata: List of metadata objects to set
            project_id: Optional project ID
            delete: Whether to delete the specified metadata (instead of
                adding/updating)

        Returns:
            Updated asset object
        """
        params = {"assetId": asset_id, "delete": str(delete).lower()}
        if project_id:
            params["projectId"] = project_id

        return self.client.request(
            "PATCH", "/v1/co/assets/metadata", params=params, json=metadata
        )

    def get_by_path(self, file_path: str) -> List[Dict]:
        """
        Get all assets with a specific path.

        Args:
            file_path: The file path to search for

        Returns:
            List of matching asset objects
        """
        return self.client.request(
            "POST", "/v1/co/assets/getByPath", json=file_path
        )

    def get_unsynced_assets(self,
                            project_id: str,
                            limit: int = 500) -> List[Dict]:
        """
        Get all unsynced assets for a project.

        Args:
            project_id: ID of the project
            limit: Maximum number of results

        Returns:
            List of unsynced asset objects
        """
        params = {"limit": limit}
        return self.client.request(
            "GET", f"/v1/co/assets/unsynced/{project_id}", params=params
        )

    def toggle_synced(
        self,
        asset_ids: List[str],
        recursive: bool = False,
        force_sync: bool = False,
    ) -> List[Dict]:
        """
        Toggle sync status for assets.

        Args:
            asset_ids: List of asset IDs to toggle
            recursive: Whether to apply recursively to assets inside a folder
            force_sync: Whether to force assets to be synced (disable toggling)

        Returns:
            List of updated asset objects
        """
        params = {
            "recursive": str(recursive).lower(),
            "forceSync": str(force_sync).lower(),
        }
        return self.client.request(
            "POST", "/v1/co/assets/toggleSynced", params=params, json=asset_ids
        )



"""
Streams module for Helmut4 client.
"""

from typing import BinaryIO, Dict, List, Optional

from ._base import BaseModule


class StreamsModule(BaseModule):
    """Module for managing streams (workflows)"""

    def get_all(self) -> List[Dict]:
        """
        Get all streams.

        Returns:
            List of stream objects
        """
        return self.client.request("GET", "/v1/streams")

    def get_by_id(self, stream_id: str) -> Dict:
        """
        Get a stream by ID.

        Args:
            stream_id: The ID of the stream

        Returns:
            Stream object
        """
        return self.client.request("GET", f"/v1/streams/id/{stream_id}")

    def get_by_event(self, event: str) -> List[Dict]:
        """
        Get streams for a specific event.

        Args:
            event: The event name (e.g., "CREATE_PROJECT")

        Returns:
            List of stream objects
        """
        return self.client.request("GET", f"/v1/streams/event/{event}")

    def get_by_tags(self, tags: List[str], design: bool = True) -> List[Dict]:
        """
        Get streams matching specified tags.

        Args:
            tags: List of tags to filter by
            design: Whether to include design information

        Returns:
            List of matching stream objects
        """
        tags_str = ",".join(tags)
        params = {"design": str(design).lower()}
        return self.client.request(
            "GET", f"/v1/streams/{tags_str}", params=params
        )

    def create(self, stream_data: Dict) -> Dict:
        """
        Create a new stream.

        Args:
            stream_data: Stream definition

        Returns:
            Created stream object
        """
        return self.client.request("POST", "/v1/streams", json=stream_data)

    def update(self, stream_data: Dict) -> Dict:
        """
        Update an existing stream.

        Args:
            stream_data: Stream data including the ID

        Returns:
            Updated stream object
        """
        return self.client.request("PUT", "/v1/streams", json=stream_data)

    def delete(self, stream_id: str) -> Dict:
        """
        Delete a stream.

        Args:
            stream_id: ID of the stream to delete

        Returns:
            Result object
        """
        return self.client.request("DELETE", f"/v1/streams/{stream_id}")

    def update_status(self, stream_id: str, status: bool) -> Dict:
        """
        Update a stream's enabled/disabled status.

        Args:
            stream_id: ID of the stream
            status: True for enabled, False for disabled

        Returns:
            Updated stream object
        """
        return self.client.request(
            "PUT", f"/v1/streams/status/{stream_id}/{str(status).lower()}"
        )

    def execute(
        self, stream_event: str, endpoint: str, content_package: Dict
    ) -> Dict:
        """
        Execute a stream.

        Args:
            stream_event: Event type triggering the stream
            endpoint: Target endpoint (e.g., "FX")
            content_package: Stream content package with execution data

        Returns:
            Stream execution result
        """
        return self.client.request(
            "POST",
            f"/v1/streams/execute/{stream_event}/{endpoint}",
            json=content_package,
        )

    def execute_custom(
        self, stream_id: str, project_id: Optional[str] = None
    ) -> Dict:
        """
        Execute a custom stream.

        Args:
            stream_id: ID of the stream to execute
            project_id: Optional project ID

        Returns:
            Execution result
        """
        if project_id:
            endpoint = f"/v1/fx/projects/execute/custom/{project_id}/{stream_id}"  # pylint: disable=line-too-long
        else:
            endpoint = f"/v1/streams/execute/custom/{stream_id}"

        return self.client.request("GET", endpoint)

    def test_stream(self, stream_data: Dict) -> str:
        """
        Test a stream without executing it.

        Args:
            stream_data: Stream definition to test

        Returns:
            Test result log
        """
        return self.client.request(
            "POST", "/v1/streams/do/test", json=stream_data
        )

    def test_path(self, path: str, os: Optional[str] = None) -> str:
        """
        Test path resolution.

        Args:
            path: Path to test
            os: Optional OS to force for resolution (WINDOWS, UNIX, AUTO)

        Returns:
            Resolved path
        """
        params = {}
        if os:
            params["os"] = os

        return self.client.request(
            "POST", "/v1/streams/do/test/path", params=params, json=path
        )

    def export(self, stream_ids: List[str]) -> bytes:
        """
        Export streams.

        Args:
            stream_ids: List of stream IDs to export

        Returns:
            Binary export file content
        """
        return self.client.request(
            "GET", f"/v1/streams/export/{','.join(stream_ids)}"
        )

    def import_streams(
        self,
        file_content: BinaryIO,
        conflict: str = "RENAME",
        purge: bool = False,
    ) -> Dict:
        """
        Import streams.

        Args:
            file_content: Binary content or file handle of the stream export
                file
            conflict: How to handle conflicts (RENAME, SKIP, OVERWRITE)
            purge: Whether to delete all existing streams first

        Returns:
            Import result
        """
        params = {"conflict": conflict, "purge": str(purge).lower()}
        files = {"file": file_content}
        return self.client.request(
            "POST", "/v1/streams/import", params=params, files=files
        )

    def get_events(self) -> List[str]:
        """
        Get all available stream events.

        Returns:
            List of event names
        """
        return self.client.request("GET", "/v1/streams/list/events")

    def get_events_by_tags(self, tags: List[str]) -> List[str]:
        """
        Get stream events filtered by tags.

        Args:
            tags: List of tags to filter by

        Returns:
            List of event names
        """
        return self.client.request(
            "GET", f"/v1/streams/list/events/{','.join(tags)}"
        )

    def get_actions(self) -> List[Dict]:
        """
        Get all available stream actions.

        Returns:
            List of action definitions
        """
        return self.client.request("GET", "/v1/streams/list/actions")

    def get_conditions(self) -> List[Dict]:
        """
        Get all available stream conditions.

        Returns:
            List of condition definitions
        """
        return self.client.request("GET", "/v1/streams/list/conditions")

    def get_outputs(self) -> List[Dict]:
        """
        Get all available stream outputs.

        Returns:
            List of output definitions
        """
        return self.client.request("GET", "/v1/streams/list/outputs")

    def get_wildcards(self) -> List[Dict]:
        """
        Get all available stream wildcards.

        Returns:
            List of wildcard definitions
        """
        return self.client.request("GET", "/v1/streams/list/wildcards")

    def resolve_string(self, resolve_package: Dict) -> Dict:
        """
        Resolve a string using the stream engine.

        Args:
            resolve_package: String resolve package with data

        Returns:
            Resolved string package
        """
        return self.client.request(
            "POST", "/v1/streams/string/resolve", json=resolve_package
        )

    def resolve_metadata(self, metadata_package: Dict) -> List[Dict]:
        """
        Resolve metadata using the stream engine.

        Args:
            metadata_package: Metadata resolve package with data

        Returns:
            List of resolved metadata objects
        """
        return self.client.request(
            "POST", "/v1/streams/metadata/resolve", json=metadata_package
        )

    def get_file_contents(self, file_chooser_data: Dict) -> Dict:
        """
        Get the contents of a folder.

        Args:
            file_chooser_data: File chooser configuration

        Returns:
            File chooser result with folder contents
        """
        return self.client.request(
            "POST", "/v1/streams/fileChooser", json=file_chooser_data
        )



"""
Languages module for Helmut4 client.
"""

from typing import Dict, List

from ._base import BaseModule


class LanguagesModule(BaseModule):
    """Module for managing languages"""

    def get_all(self) -> List[Dict]:
        """
        Get all languages.

        Returns:
            List of language objects
        """
        return self.client.request("GET", "/v1/languages")

    def create(self, language: Dict) -> Dict:
        """
        Create a new language.

        Args:
            language: Language definition

        Returns:
            Created language object
        """
        return self.client.request("POST", "/v1/languages", json=language)

    def update(self, language: Dict) -> Dict:
        """
        Update an existing language.

        Args:
            language: Language data including the ID

        Returns:
            Updated language object
        """
        return self.client.request("PUT", "/v1/languages", json=language)

    def delete(self, language_id: str) -> str:
        """
        Delete a language.

        Args:
            language_id: ID of the language to delete

        Returns:
            Success message
        """
        return self.client.request("DELETE", f"/v1/languages/{language_id}")



"""
Users module for Helmut4 client.
"""

from typing import Dict, List

from ._base import BaseModule


class UsersModule(BaseModule):
    """Module for managing users"""

    def get_all(
        self,
        group_filter: str = "All",
        limit: int = 500,
        page: int = 0
    ) -> List[Dict]:
        """
        Get all users, optionally filtered by group.

        Args:
            group_filter: Filter users by group (All, Unassigned, or a group ID)
            limit: Maximum number of results
            page: Page number for pagination

        Returns:
            List of user objects
        """
        # pylint: disable=line-too-long
        return self.client.request(
            "GET",
            f"/v1/members/users/search?groupFilter={group_filter}&limit={limit}&page={page}",
        )

    def get_by_id(self, user_id: str) -> Dict:
        """
        Get a user by ID.

        Args:
            user_id: The ID of the user

        Returns:
            User object
        """
        return self.client.request("GET", f"/v1/members/users/{user_id}")

    def get_by_name(self, username: str) -> Dict:
        """
        Get a user by username.

        Args:
            username: The username to look up

        Returns:
            User object
        """
        return self.client.request("GET", f"/v1/members/users/name/{username}")

    def create(self, user_data: Dict) -> Dict:
        """
        Create a new user.

        Args:
            user_data: User data including at minimum:
                - username: Username
                - password: Password
                - displayname: Display name
                - role: User role (Admin, User, etc.)

        Returns:
            Created user object
        """
        return self.client.request("POST", "/v1/members/users", json=user_data)

    def update(self, user_id: str, user_data: Dict) -> Dict:
        """
        Update an existing user.

        Args:
            user_id: ID of the user to update
            user_data: User data to update

        Returns:
            Updated user object
        """
        return self.client.request(
            "PUT", f"/v1/members/users/{user_id}", json=user_data
        )

    def delete(self, user_id: str) -> str:
        """
        Delete a user.

        Args:
            user_id: ID of the user to delete

        Returns:
            Success message
        """
        return self.client.request("DELETE", f"/v1/members/users/{user_id}")

    def add_to_group(self, user_id: str, group_id: str) -> Dict:
        """
        Add a user to a group.

        Args:
            user_id: ID of the user
            group_id: ID of the group

        Returns:
            Updated user object
        """
        data = {"userId": user_id, "groupId": group_id}
        return self.client.request(
            "PUT", "/v1/members/users/group/add", json=data
        )

    def remove_from_group(self, user_id: str, group_id: str) -> Dict:
        """
        Remove a user from a group.

        Args:
            user_id: ID of the user
            group_id: ID of the group

        Returns:
            Updated user object
        """
        data = {"userId": user_id, "groupId": group_id}
        return self.client.request(
            "PUT", "/v1/members/users/group/del", json=data
        )

    def change_password(
        self, user_id: str, old_password: str, new_password: str
    ) -> Dict:
        """
        Change a user's password.

        Args:
            user_id: ID of the user
            old_password: Current password
            new_password: New password

        Returns:
            Updated user object
        """
        data = {
            "id": user_id,
            "oldPassword": old_password,
            "newPassword": new_password,
        }
        return self.client.request(
            "PUT", "/v1/members/users/change/password", json=data
        )

    def get_connected_users(self) -> List[Dict]:
        """
        Get all currently connected users.

        Returns:
            List of connected user objects
        """
        return self.client.request("GET", "/v1/members/users/connected")

    def notify_user(self, username: str, message: Dict) -> Dict:
        """
        Send a notification to a user.

        Args:
            username: Username of the recipient (or * for all users)
            message: Message object with title and body

        Returns:
            Result object
        """
        return self.client.request(
            "POST", f"/v1/members/users/notification/{username}", json=message
        )

    def kick_user(self, username: str, client_session_id: str = None) -> Dict:
        """
        Kick a user from the system.

        Args:
            username: Username of the user to kick
            client_session_id: Optional client session ID for verifying the
                request

        Returns:
            Result object
        """
        params = {}
        if client_session_id:
            params = {"clientSessionId": client_session_id}

        return self.client.request(
            "GET", f"/v1/members/users/kick/{username}", params=params
        )



"""
Cronjob module for Helmut4 client.
"""

from typing import Dict, List

from ._base import BaseModule


class CronjobModule(BaseModule):
    """Module for managing cronjobs"""

    def get_all(self, limit: int = 500, page: int = 0) -> List[Dict]:
        """
        Get all cronjobs.

        Args:
            limit: Maximum number of results
            page: Page number for pagination

        Returns:
            List of cronjob objects
        """
        params = {"limit": limit, "page": page}
        return self.client.request("GET", "/v1/cronjob", params=params)

    def get_by_id(self, cronjob_id: str) -> Dict:
        """
        Get a cronjob by ID.

        Args:
            cronjob_id: The ID of the cronjob

        Returns:
            Cronjob object
        """
        return self.client.request("GET", f"/v1/cronjob/{cronjob_id}")

    def create(self, cronjob: Dict) -> Dict:
        """
        Create a new cronjob.

        Args:
            cronjob: Cronjob definition

        Returns:
            Created cronjob object
        """
        return self.client.request("POST", "/v1/cronjob", json=cronjob)

    def update(self, cronjob: Dict) -> Dict:
        """
        Update an existing cronjob.

        Args:
            cronjob: Cronjob data including the ID

        Returns:
            Updated cronjob object
        """
        return self.client.request("PUT", "/v1/cronjob", json=cronjob)

    def delete(self, cronjob_id: str) -> Dict:
        """
        Delete a cronjob.

        Args:
            cronjob_id: ID of the cronjob to delete

        Returns:
            Result object
        """
        return self.client.request("DELETE", f"/v1/cronjob/{cronjob_id}")

    def execute(self, cronjob_id: str, cron_triggered: bool = False) -> Dict:
        """
        Execute a cronjob immediately.

        Args:
            cronjob_id: ID of the cronjob to execute
            cron_triggered: Whether this execution is triggered by cron

        Returns:
            Execution result
        """
        params = {"cronTriggered": cron_triggered}
        return self.client.request(
            "GET", f"/v1/cronjob/{cronjob_id}", params=params
        )

    def explain_expression(self, cron_expression: str) -> List[str]:
        """
        Explain a cron expression by returning the next execution dates.

        Args:
            cron_expression: Cron expression to explain

        Returns:
            List of next execution dates
        """
        data = {"cronExpression": cron_expression}
        return self.client.request("POST", "/v1/cronjob/explain", json=data)



"""
Integration tests for Helmut4 client.

These tests require a running Helmut4 server instance.
Set the following environment variables to run integration tests:
- HELMUT4_HOST (default: 192.168.0.211)
- HELMUT4_PORT (default: 80)
- HELMUT4_USERNAME (default: admin)
- HELMUT4_PASSWORD (default: admin)
"""

import os

import pytest

from helmut4.client import Helmut4Client, Helmut4Error


@pytest.fixture(scope="session")
def helmut4_config():
    """Configuration for Helmut4 server."""
    return {
        "host": os.getenv("HELMUT4_HOST", "192.168.0.211"),
        "port": int(os.getenv("HELMUT4_PORT", "80")),
        "username": os.getenv("HELMUT4_USERNAME", "admin"),
        "password": os.getenv("HELMUT4_PASSWORD", "admin"),
    }


@pytest.fixture(scope="session")
def helmut4_client(helmut4_config):
    """Helmut4 client instance for integration tests."""
    base_url = f"http://{helmut4_config['host']}:{helmut4_config['port']}"

    client = Helmut4Client(
        base_url=base_url,
        username=helmut4_config["username"],
        password=helmut4_config["password"],
    )

    return client


@pytest.mark.integration
class TestHelmut4Integration:
    """Integration tests for Helmut4 client."""

    def test_authentication(self, helmut4_client):
        """Test that authentication works."""
        assert helmut4_client.token is not None
        assert len(helmut4_client.token) > 50  # JWT tokens are typically long

    def test_get_users(self, helmut4_client):
        """Test getting users from the server."""
        users = helmut4_client.users.get_all(limit=10)

        assert isinstance(users, list)
        if users:  # If there are users
            user = users[0]
            assert "id" in user
            assert "username" in user
            assert "displayname" in user

    def test_get_admin_user_by_name(self, helmut4_client, helmut4_config):
        """Test getting the admin user by username."""
        admin_user = helmut4_client.users.get_by_name(
            helmut4_config["username"]
        )

        assert admin_user["username"] == helmut4_config["username"]
        assert "id" in admin_user
        assert "role" in admin_user

    def test_search_projects(self, helmut4_client):
        """Test searching for projects."""
        projects = helmut4_client.projects.search(limit=10)

        assert isinstance(projects, list)
        # Projects list might be empty, which is fine

    def test_search_assets(self, helmut4_client):
        """Test searching for assets."""
        assets = helmut4_client.assets.search(limit=10)

        assert isinstance(assets, list)
        # Assets list might be empty, which is fine

    def test_get_all_streams(self, helmut4_client):
        """Test getting all streams."""
        streams = helmut4_client.streams.get_all()

        assert isinstance(streams, list)
        # Streams list might be empty, which is fine

    def test_invalid_authentication(self, helmut4_config):
        """Test that invalid credentials raise an error."""
        base_url = f"http://{helmut4_config['host']}:{helmut4_config['port']}"

        with pytest.raises(Helmut4Error) as exc_info:
            Helmut4Client(
                base_url=base_url,
                username="invalid_user",
                password="invalid_password",
            )

        # Should contain authentication error information
        assert exc_info.value.status_code in [401, 403]

    def test_invalid_server_url(self):
        """Test that invalid server URL raises an error."""
        with pytest.raises(Helmut4Error):
            client = Helmut4Client(
                base_url="http://nonexistent.server.invalid:12345",
                username="admin",
                password="admin",
            )
            # Try to trigger authentication
            client.authenticate()


# Skip integration tests by default
pytestmark = pytest.mark.skipif(
    os.getenv("RUN_INTEGRATION_TESTS") != "1",
    reason=
    "Integration tests require RUN_INTEGRATION_TESTS=1 environment variable",
)



"""
Unit tests for the main Helmut4Client class.
"""

from unittest.mock import Mock, patch

import pytest
import requests

from helmut4.client import Helmut4Client, Helmut4Error


class TestHelmut4ClientInit:
    """Test Helmut4Client initialization."""

    def test_init_no_auth(self, base_url):
        """Test client initialization without authentication."""
        with patch("helmut4.client.client.requests.Session"):
            client = Helmut4Client(base_url=base_url)

            assert client.base_url == base_url
            assert client.username is None
            assert client.password is None
            assert client.token is None
            assert hasattr(client, "session")

            # Verify all modules are initialized
            assert hasattr(client, "users")
            assert hasattr(client, "groups")
            assert hasattr(client, "projects")
            assert hasattr(client, "assets")
            assert hasattr(client, "jobs")
            assert hasattr(client, "streams")
            assert hasattr(client, "metadata")
            assert hasattr(client, "preferences")
            assert hasattr(client, "cronjobs")
            assert hasattr(client, "languages")
            assert hasattr(client, "licenses")

    def test_init_with_token(self, base_url, auth_credentials):
        """Test client initialization with token authentication."""
        with patch(
            "helmut4.client.client.requests.Session"
        ) as mock_session_class:
            mock_session = Mock()
            mock_session.headers = Mock()
            mock_session.headers.update = Mock()
            mock_session_class.return_value = mock_session

            client = Helmut4Client(
                base_url=base_url, token=auth_credentials["token"]
            )

            assert client.token == auth_credentials["token"]
            mock_session.headers.update.assert_called_with({
                "Authorization": f"Bearer {auth_credentials['token']}"
            })

    def test_init_with_credentials_successful_auth(
        self, base_url, auth_credentials
    ):
        """Test client initialization with username/password authentication."""
        with patch(
            "helmut4.client.client.requests.Session"
        ) as mock_session_class:
            mock_session = Mock()
            mock_session.headers = {}

            # Mock successful authentication
            mock_response = Mock()
            mock_response.raise_for_status = Mock()
            mock_response.headers = {
                "Authorization": f"Bearer {auth_credentials['token']}"
            }
            mock_response.json.return_value = {"success": True}
            mock_session.post.return_value = mock_response

            mock_session_class.return_value = mock_session

            client = Helmut4Client(
                base_url=base_url,
                username=auth_credentials["username"],
                password=auth_credentials["password"],
            )

            assert client.username == auth_credentials["username"]
            assert client.password == auth_credentials["password"]
            assert client.token == auth_credentials["token"]

            # Verify authentication was called
            mock_session.post.assert_called_once()
            auth_call = mock_session.post.call_args
            assert "/v1/members/auth/login/body" in auth_call[0][0]
            assert (
                auth_call[1]["json"]["username"] == auth_credentials["username"]
            )
            assert (
                auth_call[1]["json"]["password"] == auth_credentials["password"]
            )

    def test_base_url_trailing_slash_removed(self):
        """Test that trailing slash is removed from base URL."""
        with patch("helmut4.client.client.requests.Session"):
            client = Helmut4Client(base_url="https://example.com/")
            assert client.base_url == "https://example.com"


class TestHelmut4ClientAuthentication:
    """Test authentication methods."""

    def test_authenticate_success_token_in_header(
        self, client_no_auth, auth_credentials
    ):
        """Test successful authentication with token in header."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {
            "Authorization": f"Bearer {auth_credentials['token']}"
        }
        mock_response.json.return_value = {"success": True}

        client_no_auth.username = auth_credentials["username"]
        client_no_auth.password = auth_credentials["password"]
        client_no_auth.session.post.return_value = mock_response

        token = client_no_auth.authenticate()

        assert token == auth_credentials["token"]
        assert client_no_auth.token == auth_credentials["token"]
        # Note: In fixtures, session.headers is a real dict, so update is not mockable

    def test_authenticate_success_token_in_body(
        self, client_no_auth, auth_credentials
    ):
        """Test successful authentication with token in response body."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {}
        mock_response.json.return_value = {"token": auth_credentials["token"]}

        client_no_auth.username = auth_credentials["username"]
        client_no_auth.password = auth_credentials["password"]
        client_no_auth.session.post.return_value = mock_response

        token = client_no_auth.authenticate()

        assert token == auth_credentials["token"]
        assert client_no_auth.token == auth_credentials["token"]

    def test_authenticate_no_token_error(
        self, client_no_auth, auth_credentials
    ):
        """Test authentication failure when no token is received."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {}
        mock_response.json.return_value = {"success": True}

        client_no_auth.username = auth_credentials["username"]
        client_no_auth.password = auth_credentials["password"]
        client_no_auth.session.post.return_value = mock_response

        with pytest.raises(Helmut4Error) as exc_info:
            client_no_auth.authenticate()

        assert "no token received" in str(exc_info.value)

    def test_authenticate_http_error(self, client_no_auth, auth_credentials):
        """Test authentication failure with HTTP error."""
        error_response = Mock()
        error_response.status_code = 401
        error_response.json.return_value = {"error": "Unauthorized"}

        http_error = requests.exceptions.HTTPError()
        http_error.response = error_response

        client_no_auth.username = auth_credentials["username"]
        client_no_auth.password = auth_credentials["password"]
        client_no_auth.session.post.side_effect = http_error

        with pytest.raises(Helmut4Error) as exc_info:
            client_no_auth.authenticate()

        assert exc_info.value.status_code == 401
        assert exc_info.value.response == {"error": "Unauthorized"}

    def test_authenticate_connection_error(
        self, client_no_auth, auth_credentials
    ):
        """Test authentication failure with connection error."""
        client_no_auth.username = auth_credentials["username"]
        client_no_auth.password = auth_credentials["password"]
        client_no_auth.session.post.side_effect = (
            requests.exceptions.ConnectionError("Connection failed")
        )

        with pytest.raises(Helmut4Error) as exc_info:
            client_no_auth.authenticate()

        assert "Connection failed" in str(exc_info.value)
        assert exc_info.value.status_code is None


class TestHelmut4ClientRequest:
    """Test request method."""

    def test_request_success_json(self, client_no_auth):
        """Test successful request returning JSON."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {"Content-Type": "application/json"}
        mock_response.json.return_value = {"data": "test"}

        client_no_auth.session.request.return_value = mock_response

        result = client_no_auth.request("GET", "/test/path")

        assert result == {"data": "test"}
        client_no_auth.session.request.assert_called_once_with(
            "GET", f"{client_no_auth.base_url}/test/path"
        )

    def test_request_success_binary(self, client_no_auth):
        """Test successful request returning binary data."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {"Content-Type": "application/octet-stream"}
        mock_response.content = b"binary data"

        client_no_auth.session.request.return_value = mock_response

        result = client_no_auth.request("GET", "/download/file")

        assert result == b"binary data"

    def test_request_success_text_fallback(self, client_no_auth):
        """Test successful request falling back to text when JSON fails."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {"Content-Type": "text/plain"}
        mock_response.json.side_effect = ValueError("No JSON object")
        mock_response.text = "plain text response"

        client_no_auth.session.request.return_value = mock_response

        result = client_no_auth.request("GET", "/text/endpoint")

        assert result == "plain text response"

    def test_request_with_parameters(self, client_no_auth):
        """Test request with additional parameters."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {"Content-Type": "application/json"}
        mock_response.json.return_value = {"data": "test"}

        client_no_auth.session.request.return_value = mock_response

        result = client_no_auth.request(
            "POST",
            "/api/endpoint",
            json={"key": "value"},
            params={"filter": "test"},
        )

        assert result == {"data": "test"}
        client_no_auth.session.request.assert_called_once_with(
            "POST",
            f"{client_no_auth.base_url}/api/endpoint",
            json={"key": "value"},
            params={"filter": "test"},
        )

    def test_request_http_error_with_json_response(self, client_no_auth):
        """Test request failure with JSON error response."""
        error_response = Mock()
        error_response.status_code = 404
        error_response.json.return_value = {"error": "Not Found"}

        http_error = requests.exceptions.HTTPError()
        http_error.response = error_response

        client_no_auth.session.request.side_effect = http_error

        with pytest.raises(Helmut4Error) as exc_info:
            client_no_auth.request("GET", "/nonexistent")

        assert exc_info.value.status_code == 404
        assert exc_info.value.response == {"error": "Not Found"}

    def test_request_http_error_with_text_response(self, client_no_auth):
        """Test request failure with text error response."""
        error_response = Mock()
        error_response.status_code = 500
        error_response.json.side_effect = requests.exceptions.JSONDecodeError(
            "test", "test", 0
        )
        error_response.text = "Internal Server Error"

        http_error = requests.exceptions.HTTPError()
        http_error.response = error_response

        client_no_auth.session.request.side_effect = http_error

        with pytest.raises(Helmut4Error) as exc_info:
            client_no_auth.request("GET", "/error")

        assert exc_info.value.status_code == 500
        assert exc_info.value.response == {"error": "Internal Server Error"}

    def test_request_connection_error(self, client_no_auth):
        """Test request failure with connection error."""
        client_no_auth.session.request.side_effect = (
            requests.exceptions.ConnectionError("Connection failed")
        )

        with pytest.raises(Helmut4Error) as exc_info:
            client_no_auth.request("GET", "/test")

        assert "Connection failed" in str(exc_info.value)
        assert exc_info.value.status_code is None


class TestHelmut4ClientModuleIntegration:
    """Test integration with API modules."""

    def test_modules_have_client_reference(self, client_no_auth):
        """Test that all modules have reference to client."""
        modules = [
            client_no_auth.users,
            client_no_auth.groups,
            client_no_auth.projects,
            client_no_auth.assets,
            client_no_auth.jobs,
            client_no_auth.streams,
            client_no_auth.metadata,
            client_no_auth.preferences,
            client_no_auth.cronjobs,
            client_no_auth.languages,
            client_no_auth.licenses,
        ]

        for module in modules:
            assert hasattr(module, "client")
            assert module.client is client_no_auth

    def test_module_can_make_requests(self, client_no_auth):
        """Test that modules can make requests through client."""
        mock_response = Mock()
        mock_response.raise_for_status = Mock()
        mock_response.headers = {"Content-Type": "application/json"}
        mock_response.json.return_value = [{"id": "user1", "name": "Test User"}]

        client_no_auth.session.request.return_value = mock_response

        # Test through users module
        result = client_no_auth.users.get_all()

        assert result == [{"id": "user1", "name": "Test User"}]
        client_no_auth.session.request.assert_called_once()
        call_args = client_no_auth.session.request.call_args
        assert "users/search" in call_args[0][1]



"""
Exceptions for the Helmut4 client library.
"""


class Helmut4Error(Exception):
    """Base exception for Helmut4 API errors"""

    def __init__(
        self, message: str, status_code: int = None, response: dict = None
    ):
        self.message = message
        self.status_code = status_code
        self.response = response
        super().__init__(self.message)



"""
Unit tests for JobsModule.
"""

from unittest.mock import Mock

import pytest

from helmut4.client.modules.jobs import JobsModule


class TestJobsModule:
    """Test JobsModule functionality."""

    @pytest.fixture
    def mock_client(self):
        """Mock client for testing."""
        client = Mock()
        client.request = Mock()
        return client

    @pytest.fixture
    def jobs_module(self, mock_client):
        """JobsModule instance for testing."""
        return JobsModule(mock_client)

    def test_get_all_default_params(
        self, jobs_module, mock_client, sample_job_data
    ):
        """Test get_all with default parameters."""
        mock_client.request.return_value = [sample_job_data]

        result = jobs_module.get_all()

        assert result == [sample_job_data]
        mock_client.request.assert_called_once_with(
            "GET", "/v1/io/jobs", params={
                "limit": 500,
                "page": 0
            }
        )

    def test_get_all_custom_params(self, jobs_module, mock_client):
        """Test get_all with custom parameters."""
        mock_client.request.return_value = []

        result = jobs_module.get_all(limit=100, page=2)

        assert result == []
        mock_client.request.assert_called_once_with(
            "GET", "/v1/io/jobs", params={
                "limit": 100,
                "page": 2
            }
        )

    def test_get_by_id(self, jobs_module, mock_client, sample_job_data):
        """Test get_by_id method."""
        mock_client.request.return_value = sample_job_data

        result = jobs_module.get_by_id("job123")

        assert result == sample_job_data
        mock_client.request.assert_called_once_with("GET", "/v1/io/jobs/job123")

    def test_search_default_params(
        self, jobs_module, mock_client, sample_job_data
    ):
        """Test search with default parameters."""
        mock_client.request.return_value = [sample_job_data]

        result = jobs_module.search()

        assert result == [sample_job_data]
        mock_client.request.assert_called_once_with(
            "POST",
            "/v1/io/jobs/search",
            params={
                "limit": 500,
                "page": 0
            },
            json={},
        )

    def test_search_with_filters(self, jobs_module, mock_client):
        """Test search with filter parameters."""
        mock_client.request.return_value = []

        search_filter = {
            "query": "render",
            "status": "running",
            "userId": "user123",
        }

        result = jobs_module.search(
            search_filter=search_filter,
            limit=50,
            page=1,
        )

        assert result == []
        mock_client.request.assert_called_once_with(
            "POST",
            "/v1/io/jobs/search",
            params={
                "limit": 50,
                "page": 1
            },
            json=search_filter,
        )

    def test_create(self, jobs_module, mock_client, sample_job_data):
        """Test create method."""
        job_data = {
            "name": "Test Render Job",
            "type": "render",
            "projectId": "proj123",
        }
        mock_client.request.return_value = sample_job_data

        result = jobs_module.create(job_data)

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "POST", "/v1/io/jobs", json=job_data
        )

    def test_update_default(self, jobs_module, mock_client, sample_job_data):
        """Test update method with default parameters."""
        update_data = {"id": "job123", "progress": 75, "status": "running"}
        mock_client.request.return_value = sample_job_data

        result = jobs_module.update(update_data)

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "PUT",
            "/v1/io/jobs",
            params={"postpone": "false"},
            json=update_data
        )

    def test_update_with_postpone(
        self, jobs_module, mock_client, sample_job_data
    ):
        """Test update method with postpone option."""
        update_data = {"id": "job123", "progress": 100, "status": "completed"}
        mock_client.request.return_value = sample_job_data

        result = jobs_module.update(update_data, postpone=True)

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "PUT", "/v1/io/jobs", params={"postpone": "true"}, json=update_data
        )

    def test_delete(self, jobs_module, mock_client):
        """Test delete method."""
        mock_client.request.return_value = "Job deleted successfully"

        result = jobs_module.delete("job123")

        assert result == "Job deleted successfully"
        mock_client.request.assert_called_once_with(
            "DELETE", "/v1/io/jobs/job123"
        )

    def test_cancel(self, jobs_module, mock_client, sample_job_data):
        """Test cancel method."""
        job_data = {"id": "job123"}
        mock_client.request.return_value = sample_job_data

        result = jobs_module.cancel(job_data)

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "POST", "/v1/io/jobs/cancel", json=job_data
        )

    def test_update_logs(self, jobs_module, mock_client, sample_job_data):
        """Test update_logs method."""
        log_data = {
            "message": "Processing frame 100/200",
            "level": "info",
            "timestamp": "2023-01-01T12:00:00Z",
        }
        mock_client.request.return_value = sample_job_data

        result = jobs_module.update_logs("job123", log_data)

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "PUT", "/v1/io/jobs/logs/job123", json=log_data
        )

    def test_set_priority(self, jobs_module, mock_client, sample_job_data):
        """Test set_priority method."""
        job_data = {"id": "job123", "priority": 8}
        mock_client.request.return_value = sample_job_data

        result = jobs_module.set_priority("job123", job_data)

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "PUT", "/v1/io/jobs/priority/job123", json=job_data
        )

    def test_set_finish_date(self, jobs_module, mock_client, sample_job_data):
        """Test set_finish_date method."""
        mock_client.request.return_value = sample_job_data

        result = jobs_module.set_finish_date("job123")

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "PATCH", "/v1/io/jobs/job123/finish"
        )

    def test_request_processing(
        self, jobs_module, mock_client, sample_job_data
    ):
        """Test request_processing method."""
        mock_client.request.return_value = sample_job_data

        result = jobs_module.request_processing()

        assert result == sample_job_data
        mock_client.request.assert_called_once_with(
            "GET", "/v1/io/jobs/process"
        )

    def test_purge_old_jobs_default(self, jobs_module, mock_client):
        """Test purge_old_jobs with default parameters."""
        mock_client.request.return_value = {
            "deleted": 25,
            "message": "Jobs purged",
        }

        result = jobs_module.purge_old_jobs()

        assert result == {"deleted": 25, "message": "Jobs purged"}
        mock_client.request.assert_called_once_with("GET", "/v1/io/jobs/purge")

    def test_purge_old_jobs_no_params(self, jobs_module, mock_client):
        """Test purge_old_jobs method."""
        mock_client.request.return_value = {
            "deleted": 10,
            "message": "Jobs purged",
        }

        result = jobs_module.purge_old_jobs()

        assert result == {"deleted": 10, "message": "Jobs purged"}
        mock_client.request.assert_called_once_with("GET", "/v1/io/jobs/purge")



"""
Unit tests for StreamsModule.
"""

from unittest.mock import Mock

import pytest

from helmut4.client.modules.streams import StreamsModule


class TestStreamsModule:
    """Test StreamsModule functionality."""

    @pytest.fixture
    def mock_client(self):
        """Mock client for testing."""
        client = Mock()
        client.request = Mock()
        return client

    @pytest.fixture
    def streams_module(self, mock_client):
        """StreamsModule instance for testing."""
        return StreamsModule(mock_client)

    def test_get_all(self, streams_module, mock_client, sample_stream_data):
        """Test get_all method."""
        mock_client.request.return_value = [sample_stream_data]

        result = streams_module.get_all()

        assert result == [sample_stream_data]
        mock_client.request.assert_called_once_with("GET", "/v1/streams")

    def test_get_by_id(self, streams_module, mock_client, sample_stream_data):
        """Test get_by_id method."""
        mock_client.request.return_value = sample_stream_data

        result = streams_module.get_by_id("stream123")

        assert result == sample_stream_data
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/id/stream123"
        )

    def test_get_by_event(
        self, streams_module, mock_client, sample_stream_data
    ):
        """Test get_by_event method."""
        mock_client.request.return_value = [sample_stream_data]

        result = streams_module.get_by_event("onAssetCreate")

        assert result == [sample_stream_data]
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/event/onAssetCreate"
        )

    def test_get_by_tags(self, streams_module, mock_client, sample_stream_data):
        """Test get_by_tags method."""
        mock_client.request.return_value = [sample_stream_data]

        result = streams_module.get_by_tags(["automation", "workflow"])

        assert result == [sample_stream_data]
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/automation,workflow", params={"design": "true"}
        )

    def test_create(self, streams_module, mock_client, sample_stream_data):
        """Test create method."""
        stream_data = {
            "name": "New Stream",
            "event": "onAssetCreate",
            "enabled": True,
            "conditions": [],
            "actions": [],
        }
        mock_client.request.return_value = sample_stream_data

        result = streams_module.create(stream_data)

        assert result == sample_stream_data
        mock_client.request.assert_called_once_with(
            "POST", "/v1/streams", json=stream_data
        )

    def test_update(self, streams_module, mock_client, sample_stream_data):
        """Test update method."""
        update_data = {
            "id": "stream123",
            "enabled": False,
            "name": "Updated Stream",
        }
        mock_client.request.return_value = sample_stream_data

        result = streams_module.update(update_data)

        assert result == sample_stream_data
        mock_client.request.assert_called_once_with(
            "PUT", "/v1/streams", json=update_data
        )

    def test_delete(self, streams_module, mock_client):
        """Test delete method."""
        mock_client.request.return_value = "Stream deleted successfully"

        result = streams_module.delete("stream123")

        assert result == "Stream deleted successfully"
        mock_client.request.assert_called_once_with(
            "DELETE", "/v1/streams/stream123"
        )

    def test_update_status_enable(
        self, streams_module, mock_client, sample_stream_data
    ):
        """Test update_status to enable stream."""
        mock_client.request.return_value = sample_stream_data

        result = streams_module.update_status("stream123", True)

        assert result == sample_stream_data
        mock_client.request.assert_called_once_with(
            "PUT", "/v1/streams/status/stream123/true"
        )

    def test_update_status_disable(
        self, streams_module, mock_client, sample_stream_data
    ):
        """Test update_status to disable stream."""
        mock_client.request.return_value = sample_stream_data

        result = streams_module.update_status("stream123", False)

        assert result == sample_stream_data
        mock_client.request.assert_called_once_with(
            "PUT", "/v1/streams/status/stream123/false"
        )

    def test_execute(self, streams_module, mock_client):
        """Test execute method."""
        content_package = {"assetId": "asset123", "projectId": "proj123"}
        mock_client.request.return_value = {"success": True, "executed": True}

        result = streams_module.execute("onAssetCreate", "FX", content_package)

        assert result == {"success": True, "executed": True}
        mock_client.request.assert_called_once_with(
            "POST",
            "/v1/streams/execute/onAssetCreate/FX",
            json=content_package
        )

    def test_execute_custom(self, streams_module, mock_client):
        """Test execute_custom method."""
        mock_client.request.return_value = {"success": True}

        result = streams_module.execute_custom("stream123")

        assert result == {"success": True}
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/execute/custom/stream123"
        )

    def test_test_stream(self, streams_module, mock_client):
        """Test test_stream method."""
        stream_data = {
            "id": "stream123",
            "name": "Test Stream",
            "event": "onAssetCreate",
            "actions": [],
        }
        mock_client.request.return_value = "Test completed successfully"

        result = streams_module.test_stream(stream_data)

        assert result == "Test completed successfully"
        mock_client.request.assert_called_once_with(
            "POST", "/v1/streams/do/test", json=stream_data
        )

    def test_test_path(self, streams_module, mock_client):
        """Test test_path method."""
        mock_client.request.return_value = "/resolved/path/file.mp4"

        result = streams_module.test_path("/original/path/{asset.name}")

        assert result == "/resolved/path/file.mp4"
        mock_client.request.assert_called_once_with(
            "POST",
            "/v1/streams/do/test/path",
            params={},
            json="/original/path/{asset.name}",
        )

    def test_export(self, streams_module, mock_client):
        """Test export method."""
        mock_client.request.return_value = b"exported streams data"

        result = streams_module.export(["stream123", "stream456"])

        assert result == b"exported streams data"
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/export/stream123,stream456"
        )

    def test_import_streams_default(self, streams_module, mock_client):
        """Test import_streams with default parameters."""
        file_data = b"streams import data"
        mock_client.request.return_value = {"imported": 5, "skipped": 1}

        result = streams_module.import_streams(file_data)

        assert result == {"imported": 5, "skipped": 1}
        mock_client.request.assert_called_once_with(
            "POST",
            "/v1/streams/import",
            params={
                "conflict": "RENAME",
                "purge": "false"
            },
            files={"file": file_data},
        )

    def test_import_streams_with_overwrite(self, streams_module, mock_client):
        """Test import_streams with overwrite option."""
        file_data = b"streams import data"
        mock_client.request.return_value = {"imported": 3, "overwritten": 2}

        result = streams_module.import_streams(
            file_data, conflict="OVERWRITE", purge=True
        )

        assert result == {"imported": 3, "overwritten": 2}
        mock_client.request.assert_called_once_with(
            "POST",
            "/v1/streams/import",
            params={
                "conflict": "OVERWRITE",
                "purge": "true"
            },
            files={"file": file_data},
        )

    def test_get_events(self, streams_module, mock_client):
        """Test get_events method."""
        events = [
            {
                "name": "onAssetCreate",
                "description": "When asset is created"
            },
            {
                "name": "onAssetUpdate",
                "description": "When asset is updated"
            },
        ]
        mock_client.request.return_value = events

        result = streams_module.get_events()

        assert result == events
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/list/events"
        )

    def test_get_events_by_tags(self, streams_module, mock_client):
        """Test get_events_by_tags method."""
        events = [{"name": "onAssetCreate", "tags": ["asset", "create"]}]
        mock_client.request.return_value = events

        result = streams_module.get_events_by_tags(["asset", "create"])

        assert result == events
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/list/events/asset,create"
        )

    def test_resolve_string(self, streams_module, mock_client):
        """Test resolve_string method."""
        resolve_data = {
            "string": "Asset {asset.name} created",
            "context": {
                "asset": {
                    "name": "test.mp4"
                }
            },
        }
        mock_client.request.return_value = {
            "resolved": "Asset test.mp4 created"
        }

        result = streams_module.resolve_string(resolve_data)

        assert result == {"resolved": "Asset test.mp4 created"}
        mock_client.request.assert_called_once_with(
            "POST", "/v1/streams/string/resolve", json=resolve_data
        )

    def test_get_file_contents(self, streams_module, mock_client):
        """Test get_file_contents method."""
        file_chooser_data = {"path": "/projects/test", "filter": "*.mp4"}
        mock_client.request.return_value = {
            "files": ["file1.mp4", "file2.avi"],
            "folders": ["subfolder1", "subfolder2"],
        }

        result = streams_module.get_file_contents(file_chooser_data)

        assert result == {
            "files": ["file1.mp4", "file2.avi"],
            "folders": ["subfolder1", "subfolder2"],
        }
        mock_client.request.assert_called_once_with(
            "POST", "/v1/streams/fileChooser", json=file_chooser_data
        )

    def test_get_actions(self, streams_module, mock_client):
        """Test get_actions method."""
        actions = [
            {
                "id": "action1",
                "name": "Log Message"
            },
            {
                "id": "action2",
                "name": "Send Email"
            },
        ]
        mock_client.request.return_value = actions

        result = streams_module.get_actions()

        assert result == actions
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/list/actions"
        )

    def test_get_conditions(self, streams_module, mock_client):
        """Test get_conditions method."""
        conditions = [
            {
                "id": "cond1",
                "name": "File Extension"
            },
            {
                "id": "cond2",
                "name": "File Size"
            },
        ]
        mock_client.request.return_value = conditions

        result = streams_module.get_conditions()

        assert result == conditions
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/list/conditions"
        )

    def test_get_outputs(self, streams_module, mock_client):
        """Test get_outputs method."""
        outputs = [
            {
                "id": "out1",
                "name": "File Output"
            },
            {
                "id": "out2",
                "name": "Database Output"
            },
        ]
        mock_client.request.return_value = outputs

        result = streams_module.get_outputs()

        assert result == outputs
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/list/outputs"
        )

    def test_get_wildcards(self, streams_module, mock_client):
        """Test get_wildcards method."""
        wildcards = [
            {
                "id": "wild1",
                "name": "{asset.name}"
            },
            {
                "id": "wild2",
                "name": "{project.id}"
            },
        ]
        mock_client.request.return_value = wildcards

        result = streams_module.get_wildcards()

        assert result == wildcards
        mock_client.request.assert_called_once_with(
            "GET", "/v1/streams/list/wildcards"
        )

    def test_resolve_metadata(self, streams_module, mock_client):
        """Test resolve_metadata method."""
        metadata_package = {
            "metadata": [{
                "key": "title",
                "value": "{asset.name}"
            }],
            "context": {
                "asset": {
                    "name": "test.mp4"
                }
            },
        }
        mock_client.request.return_value = [{
            "key": "title",
            "value": "test.mp4"
        }]

        result = streams_module.resolve_metadata(metadata_package)

        assert result == [{"key": "title", "value": "test.mp4"}]
        mock_client.request.assert_called_once_with(
            "POST", "/v1/streams/metadata/resolve", json=metadata_package
        )



"""
Projects module for Helmut4 client.
"""

from typing import BinaryIO, Dict, List, Optional

from ._base import BaseModule


class ProjectsModule(BaseModule):
    """Module for managing projects"""

    def search(
        self,
        search_filter: Optional[Dict] = None,
        limit: int = 500,
        page: int = 0,
    ) -> List[Dict]:
        """
        Search for projects based on a filter.

        Args:
            search_filter: Search filter criteria
            limit: Maximum number of results
            page: Page number for pagination

        Returns:
            List of matching project objects
        """
        params = {"limit": limit, "page": page}
        return self.client.request(
            "POST",
            "/v1/fx/projects/search",
            params=params,
            json=search_filter or {},
        )

    def get_by_id(self, project_id: str) -> Dict:
        """
        Get a project by ID.

        Args:
            project_id: The ID of the project

        Returns:
            Project object
        """
        return self.client.request("GET", f"/v1/fx/projects/{project_id}")

    def create(self, project_data: Dict) -> Dict:
        """
        Create a new project.

        Args:
            project_data: Project data

        Returns:
            Created project object
        """
        return self.client.request("POST", "/v1/fx/projects", json=project_data)

    def update(self, project_data: Dict) -> Dict:
        """
        Update an existing project.

        Args:
            project_data: Project data including the ID

        Returns:
            Updated project object
        """
        return self.client.request("PUT", "/v1/fx/projects", json=project_data)

    def delete(self, project_id: str, suppress_event: bool = False) -> Dict:
        """
        Delete a project.

        Args:
            project_id: ID of the project to delete
            suppress_event: Whether to suppress the UNLOCK_PROJECT event

        Returns:
            Result object
        """
        params = {}
        if suppress_event:
            params = {"suppressEvent": "true"}

        return self.client.request(
            "DELETE", f"/v1/fx/projects/{project_id}", params=params
        )

    def open(self, project_id: str) -> Dict:
        """
        Open a project.

        Args:
            project_id: ID of the project to open

        Returns:
            Project object
        """
        return self.client.request("GET", f"/v1/fx/projects/open/{project_id}")

    def duplicate(self, project_data: Dict) -> Dict:
        """
        Duplicate an existing project.

        Args:
            project_data: Source project data with modifications for the
                duplicate

        Returns:
            Created project object
        """
        return self.client.request(
            "POST", "/v1/fx/projects/duplicate", json=project_data
        )

    def download(self, project_id: str) -> bytes:
        """
        Download a project file.

        Args:
            project_id: ID of the project to download

        Returns:
            Binary project file content
        """
        return self.client.request(
            "GET", f"/v1/fx/projects/download/{project_id}"
        )

    def upload(self, project_id: str, file_content: BinaryIO) -> Dict:
        """
        Upload a project file.

        Args:
            project_id: ID of the project to update
            file_content: Binary content or file handle of the project file

        Returns:
            Updated project object
        """
        files = {"file": file_content}
        return self.client.request(
            "PUT", f"/v1/fx/projects/upload/{project_id}", files=files
        )

    def set_status(
        self,
        project_id: str,
        status: str,
        suppress_unlock_event: bool = False
    ) -> Dict:
        """
        Set a project's status.

        Args:
            project_id: ID of the project
            status: New status (e.g., "LOCKED", "UNLOCKED")
            suppress_unlock_event: Whether to suppress the AUTO_IMPORT event

        Returns:
            Updated project object
        """
        params = {}
        if suppress_unlock_event:
            params = {"suppressUnlockEvent": "true"}

        return self.client.request(
            "PUT",
            f"/v1/fx/projects/status/{project_id}",
            params=params,
            json={"status": status},
        )

    def get_templates(self, group: str, category: str) -> List[str]:
        """
        Get available templates for a group and category.

        Args:
            group: Project group
            category: Project category

        Returns:
            List of template names
        """
        return self.client.request(
            "GET", f"/v1/fx/templates/{group}/{category}"
        )

    def import_project(self, file_content: BinaryIO) -> str:
        """
        Import a project file.

        Args:
            file_content: Binary content or file handle of the project file

        Returns:
            Temporary path to the imported project
        """
        files = {"file": file_content}
        return self.client.request(
            "POST", "/v1/fx/projects/import", files=files
        )
